'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends8 = require('babel-runtime/helpers/extends');

var _extends9 = _interopRequireDefault(_extends8);

exports.subscribeHandler = subscribeHandler;
exports.unsubscribeHandler = unsubscribeHandler;
exports.mergeHandler = mergeHandler;
exports.provideViewModelsHandler = provideViewModelsHandler;
exports.createMap = createMap;
exports.viewModelEventHandler = viewModelEventHandler;
exports.default = createViewModelsReducer;

var _action_types = require('./action_types');

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function subscribeHandler(_ref, state, _ref2) {
  var subscribers = _ref.subscribers,
      viewModels = _ref.viewModels;
  var viewModelName = _ref2.viewModelName,
      aggregateId = _ref2.aggregateId;

  var key = (0, _util.getKey)(viewModelName, aggregateId);

  if (subscribers[key]) {
    subscribers[key]++;
    return state;
  }

  subscribers[key] = 1;

  return (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, viewModelName, (0, _extends9.default)({}, state[viewModelName], (0, _defineProperty3.default)({}, aggregateId, (viewModels.find(function (_ref3) {
    var name = _ref3.name;
    return viewModelName === name;
  }).projection.Init || function () {})()))));
}

function unsubscribeHandler(_ref4, state, _ref5) {
  var subscribers = _ref4.subscribers;
  var viewModelName = _ref5.viewModelName,
      aggregateId = _ref5.aggregateId;

  var key = (0, _util.getKey)(viewModelName, aggregateId);

  if (subscribers[key] > 1) {
    subscribers[key]--;
    return state;
  }

  subscribers[key] = 0;

  var nextViewModel = (0, _extends9.default)({}, state[viewModelName]);
  delete nextViewModel[aggregateId];

  return (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, viewModelName, nextViewModel));
}

function mergeHandler(_, state, _ref6) {
  var viewModelName = _ref6.viewModelName,
      aggregateId = _ref6.aggregateId,
      actionState = _ref6.state;

  return (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, viewModelName, (0, _extends9.default)({}, state[viewModelName], (0, _defineProperty3.default)({}, aggregateId, actionState))));
}

function provideViewModelsHandler(context, state, _ref7) {
  var viewModels = _ref7.viewModels;
  var handlers = context.handlers,
      initialState = context.initialState;

  context.viewModels = viewModels;

  delete handlers[_action_types.PROVIDE_VIEW_MODELS];

  handlers[_action_types.SUBSCRIBE_VIEWMODEL] = subscribeHandler.bind(null, context);

  handlers[_action_types.UNSUBSCRIBE_VIEWMODEL] = unsubscribeHandler.bind(null, context);

  handlers[_action_types.MERGE] = mergeHandler.bind(null, context);

  viewModels.forEach(function (_ref8) {
    var viewModelName = _ref8.name;

    initialState[viewModelName] = (0, _extends9.default)({}, initialState[viewModelName]);
  });

  var map = createMap(viewModels);

  (0, _keys2.default)(map).forEach(function (eventType) {
    handlers[eventType] = viewModelEventHandler.bind(null, map[eventType]);
  });

  return initialState;
}

function createMap(viewModels) {
  return viewModels.reduce(function (acc, _ref9) {
    var viewModelName = _ref9.name,
        projection = _ref9.projection;

    var handlers = (0, _extends9.default)({}, projection);
    delete handlers.Init;
    (0, _keys2.default)(handlers).forEach(function (eventType) {
      if (!acc[eventType]) {
        acc[eventType] = {};
      }

      acc[eventType][viewModelName] = projection[eventType];
    });
    return acc;
  }, {});
}

function viewModelEventHandler(viewModels, state, action) {
  var nextState = (0, _extends9.default)({}, state);

  (0, _keys2.default)(viewModels).forEach(function (viewModelName) {
    if (!state[viewModelName]) return;

    if (state[viewModelName].hasOwnProperty('*')) {
      var _viewModelState = state[viewModelName]['*'];

      var _result = viewModels[viewModelName](_viewModelState, action);

      nextState[viewModelName] = (0, _extends9.default)({}, nextState[viewModelName], {
        '*': _result
      });
    }

    if (!state[viewModelName].hasOwnProperty(action.aggregateId)) return;

    var viewModelState = state[viewModelName][action.aggregateId];

    var result = viewModels[viewModelName](viewModelState, action);

    nextState[viewModelName] = (0, _extends9.default)({}, nextState[viewModelName], (0, _defineProperty3.default)({}, action.aggregateId, result));
  });

  return nextState;
}

function createViewModelsReducer() {
  var context = {
    initialState: {},
    handlers: {},
    subscribers: {}
  };

  context.handlers[_action_types.PROVIDE_VIEW_MODELS] = provideViewModelsHandler.bind(null, context);

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    var eventHandler = context.handlers[action.type];

    if (eventHandler) {
      return eventHandler(state, action);
    }

    return state;
  };
}