'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.checkRequiredFields = checkRequiredFields;
exports.getRootableUrl = getRootableUrl;
exports.getKey = getKey;
exports.delay = delay;
exports.makeLateResolvingPromise = makeLateResolvingPromise;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rootPath = typeof process !== 'undefined' && typeof process.env !== 'undefined' && process.env['ROOT_PATH'] ? '/' + process.env['ROOT_PATH'] : '';

function checkRequiredFields(obj, beforeWarnings, afterWarnings) {
  var warningMessages = (0, _keys2.default)(obj).map(function (fieldName) {
    return obj[fieldName] ? null : 'The \'' + fieldName + '\' field is required';
  }).filter(function (msg) {
    return msg;
  });

  var shouldWarningsBePrinted = warningMessages.length > 0;

  if (shouldWarningsBePrinted) {
    // eslint-disable-next-line no-console
    console.warn([beforeWarnings].concat((0, _toConsumableArray3.default)(warningMessages), [afterWarnings]).filter(function (line) {
      return line;
    }).join('\n'));
  }

  return !shouldWarningsBePrinted;
}

function getRootableUrl(path) {
  if (/^https?:\/\//.test(path)) {
    return path;
  }
  return rootPath + '/' + path.replace(/^\//, '');
}

function getKey(viewModel, aggregateId) {
  return viewModel + ':' + aggregateId;
}

function delay(timeout) {
  return new _promise2.default(function (resolve) {
    return setTimeout(resolve, timeout);
  });
}

function makeLateResolvingPromise() {
  if (arguments.length > 1) {
    throw new Error('Make late-resolved promise accepts only zero on one argument');
  }

  var resolvePromise = null;
  var promise = new _promise2.default(function (resolve) {
    return resolvePromise = resolve;
  });
  Object.defineProperty(promise, 'resolve', { value: resolvePromise });

  if (arguments.length === 1) {
    resolvePromise(arguments.length <= 0 ? undefined : arguments[0]);
  }

  return promise;
}